<?php

declare(strict_types=1);

namespace Buddy\Repman\Tests\Unit\Service\Organization;

use Buddy\Repman\Query\User\Model\Organization\Member;
use Buddy\Repman\Security\Model\User;
use Buddy\Repman\Service\Organization\OrganizationVoter;
use Buddy\Repman\Tests\MotherObject\Query\OrganizationMother;
use Buddy\Repman\Tests\MotherObject\Security\UserMother;
use PHPUnit\Framework\TestCase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Authentication\Token\AnonymousToken;
use Symfony\Component\Security\Core\Authentication\Token\UsernamePasswordToken;
use Symfony\Component\Security\Core\Authorization\Voter\Voter;

final class OrganizationVoterTest extends TestCase
{
    private OrganizationVoter $voter;
    private UsernamePasswordToken $token;
    private string $userId;

    protected function setUp(): void
    {
        $this->userId = 'some-id';
        $this->token = new UsernamePasswordToken(UserMother::withOrganizations($this->userId, [
            new User\Organization('repman', 'name', 'owner'),
            new User\Organization('buddy', 'name', 'member'),
        ]), 'password', 'key');
        $this->voter = new OrganizationVoter();
    }

    public function testSupportOnlyUserInstance(): void
    {
        self::assertEquals(Voter::ACCESS_DENIED, $this->voter->vote(
            new AnonymousToken('secret', 'anon', []),
            'any',
            ['ROLE_ORGANIZATION_MEMBER']
        ));
    }

    public function testDenyIfNoSubject(): void
    {
        self::assertEquals(Voter::ACCESS_DENIED, $this->voter->vote(
            $this->token,
            null,
            ['ROLE_ORGANIZATION_MEMBER']
        ));
    }

    public function testAccessForOwnerWithOrganizationReadModel(): void
    {
        self::assertEquals(Voter::ACCESS_GRANTED, $this->voter->vote(
            $this->token,
            OrganizationMother::withMember(new Member($this->userId, 'email', 'owner')),
            ['ROLE_ORGANIZATION_OWNER']
        ));

        self::assertEquals(Voter::ACCESS_DENIED, $this->voter->vote(
            $this->token,
            OrganizationMother::withMember(new Member($this->userId, 'email', 'member')),
            ['ROLE_ORGANIZATION_OWNER']
        ));

        // other organization
        self::assertEquals(Voter::ACCESS_DENIED, $this->voter->vote(
            $this->token,
            OrganizationMother::withMember(new Member('other-id', 'email', 'member')),
            ['ROLE_ORGANIZATION_OWNER']
        ));

        self::assertEquals(Voter::ACCESS_DENIED, $this->voter->vote(
            $this->token,
            OrganizationMother::withMember(new Member('other-id', 'email', 'member')),
            ['ROLE_ORGANIZATION_MEMBER']
        ));
    }

    public function testAccessForOwnerWithRequest(): void
    {
        // owner organization
        self::assertEquals(Voter::ACCESS_GRANTED, $this->voter->vote(
            $this->token,
            new Request([], ['organization' => 'repman']),
            ['ROLE_ORGANIZATION_OWNER']
        ));

        self::assertEquals(Voter::ACCESS_GRANTED, $this->voter->vote(
            $this->token,
            new Request([], ['organization' => 'repman']),
            ['ROLE_ORGANIZATION_MEMBER']
        ));

        // member organization
        self::assertEquals(Voter::ACCESS_DENIED, $this->voter->vote(
            $this->token,
            new Request([], ['organization' => 'buddy']),
            ['ROLE_ORGANIZATION_OWNER']
        ));

        self::assertEquals(Voter::ACCESS_GRANTED, $this->voter->vote(
            $this->token,
            new Request([], ['organization' => 'buddy']),
            ['ROLE_ORGANIZATION_MEMBER']
        ));

        // other organization
        self::assertEquals(Voter::ACCESS_DENIED, $this->voter->vote(
            $this->token,
            new Request([], ['organization' => 'other']),
            ['ROLE_ORGANIZATION_MEMBER']
        ));

        self::assertEquals(Voter::ACCESS_DENIED, $this->voter->vote(
            $this->token,
            new Request([], ['organization' => 'other']),
            ['ROLE_ORGANIZATION_OWNER']
        ));
    }
}
